@@ -30,6 +30,13 @@
 
 #include "grab-ng.h"
 
+#ifdef HAVE_LIBV4L
+#include <libv4l2.h>
+#else
+#define v4l2_open open
+#define v4l2_close close
+#endif
+
 int  ng_debug          = 0;
 int  ng_log_bad_stream = 0;
 int  ng_log_resync     = 0;
@@ -64,6 +71,8 @@ const unsigned int ng_vfmt_to_depth[VIDEO_FMT_COUNT] = {
     0,		     /* JPEG     */
     16,		     /* UYVY     */
     0,		     /* MPEG     */
+    24,		     /* BAYER	*/
+    24,		     /* S910	*/
 };
 
 const char* ng_vfmt_to_desc[VIDEO_FMT_COUNT] = {
@@ -87,6 +96,8 @@ const char* ng_vfmt_to_desc[VIDEO_FMT_COUNT] = {
     "JPEG (JFIF)",
     "16 bit YUV 4:2:2 (packed, UYVY)",
     "MPEG video",
+    "Sequential Bayer (BA81)",
+    "SN9C102 Driver Compressed Format (S910)",  
 };
 
 /* --------------------------------------------------------------------- */
@@ -849,7 +860,7 @@ int ng_dev_users(struct ng_devstate *dev)
     return dev->refcount;
 }
 
-int ng_chardev_open(char *device, int flags, int major, int complain)
+int ng_chardev_open(char *device, int flags, int major, int complain, int is_v4l2)
 {
     struct stat st;
     int fd = -1;
@@ -859,14 +870,22 @@ int ng_chardev_open(char *device, int flags, int major, int complain)
 	    fprintf(stderr,"%s: not below /dev\n",device);
 	goto err;
     }
-    if (-1 == (fd = open(device, flags))) {
-	if (complain)
-	    fprintf(stderr,"open(%s): %s\n",device,strerror(errno));
-	goto err;
+    if (1 == is_v4l2) {
+	if (-1 == (fd = v4l2_open(device, flags))) {
+		if (complain)
+		    fprintf(stderr,"open(%s): %s\n",device,strerror(errno));
+		goto err;
+	    }
+    } else {
+        if (-1 == (fd = open(device, flags))) {
+                if (complain)
+                    fprintf(stderr,"open(%s): %s\n",device,strerror(errno));
+                goto err;
+            }
     }
     if (-1 == fstat(fd,&st)) {
 	if (complain)
-	    fprintf(stderr,"fstat(%s): %s\n",device,strerror(errno));
+	    	fprintf(stderr,"fstat(%s): %s\n",device,strerror(errno));
 	goto err;
     }
     if (!S_ISCHR(st.st_mode)) {
@@ -885,7 +904,10 @@ int ng_chardev_open(char *device, int flags, int major, int complain)
 
  err:
     if (-1 != fd)
-	close(fd);
+	if (1 == is_v4l2)
+		v4l2_close(fd);
+	else
+		close(fd);
     return -1;
 }
 
@@ -1142,7 +1164,6 @@ static int ng_plugins(char *dirname)
 
 	if (NULL == (initcall = dlsym(plugin,"ng_plugin_init"))) {
 	    if (NULL == (initcall = dlsym(plugin,"_ng_plugin_init"))) {
-		fprintf(stderr,"dlsym[%s]: %s\n",filename,dlerror());
 		continue;
 	    }
 	}
@@ -1186,6 +1207,8 @@ ng_init(void)
     count += ng_plugins("./utils/linux/capture/libng/contrib-plugins");
 
  
+    /*
     if (0 == count)
 	fprintf(stderr,"WARNING: no plugins found [%s]\n",LIBDIR);
+    */
 }
