@@ -46,6 +46,10 @@
 
 
 		Author : Arieh Schneier ( lio_lion - lio_lion@user.sourceforge.net)
+
+		In order to be able to use UNICODE functions with Win9x we must use the unicows system
+		http://libunicows.sourceforge.net/ provides a free implementation of the lib that is compatible with opencow and unicows
+
 */
 
 //HWND hWnd = Tk_GetHWND(Tk_WindowId((Tk_Window) clientData));
@@ -61,6 +65,7 @@ static int Tk_WinLoadFile (ClientData clientData,
 	WCHAR *file = NULL;
 	Tcl_Obj *argsObj = NULL;
 	WCHAR* argsStr = NULL;
+	int res;
 
 	// We verify the arguments, we must have one arg, not more
 	if( objc < 2) {
@@ -82,11 +87,19 @@ static int Tk_WinLoadFile (ClientData clientData,
 		argsStr = Tcl_GetUnicode(argsObj);
 	}
 
-	if ((int) ShellExecute(NULL,L"open", file, argsStr, NULL, SW_SHOWNORMAL) <= 32) {
+	res = (int) ShellExecute(NULL, L"open", file, argsStr, NULL, SW_SHOWNORMAL);
+	if (res <= 32) {
+		Tcl_Obj *result = Tcl_NewStringObj("Unable to open file : ", strlen("Unable to open file : "));
+		Tcl_AppendUnicodeToObj(result, file, lstrlen(file));
+		Tcl_AppendToObj(result, " " , strlen(" "));
+		Tcl_AppendUnicodeToObj(result, argsStr, lstrlen(argsStr));
+		Tcl_AppendToObj(result, " : " , strlen(" : "));
+		Tcl_AppendObjToObj(result, Tcl_NewIntObj(res));
+
+		Tcl_SetObjResult(interp, result);
 		return TCL_ERROR;
 	}
 
-
 	return TCL_OK;
 }
 
